'
'Bad Code :D
'

Const UIEventLeftMouseDown		:Int = 1
Const UIEventLeftMouseup		:Int = 2
Const UIEventLeftMouseDragged	:Int = 3
Const UIEventRightMouseDown		:Int = 7
Const UIEventRightMouseup		:Int = 8
Const UIEventRightMouseDragged	:Int = 9
Const UIEventMouseMoved		:Int = 10
Const UIEventMouseEntered		:Int = 11
Const UIEventMouseLeft			:Int = 12
Const UIEventMouseWheel		:Int = 13
Const UIEventKeyDown			:Int = 14
Const UIEventKeyUp				:Int = 15
Const UIEventCharacter			:Int = 16

Const UILeftMouseButton		:Byte = %001
Const UIRightMouseButton		:Byte = %010
Const UIOtherMouseButton		:Byte = %100


Rem
bbdoc:
EndRem
Type UIEvent Extends StdObject Final

	Global _dlocx:Int, _dlocy:Int, _dlocz:Int
	Global _locx:Int, _locy:Int, _locz:Int
	Global _d1:Int, _d2:Int, _d3:Int

	Field _type:Int
	Field _time:Int
	
	Field _keycode:Byte
	Field _mousebuttons:Byte
	Field _mousex:Float, _mousey:Float, _mousez:Float
	
	'------
	'create
	'------
	
	Function Create:UIEvent( type_:Int=0 )
		Local ret_:UIEvent = New UIEvent
			ret_.initWithType(type_)
		Return ret_
	EndFunction
	
	Method initWithType( type_:Int )
		_type = type_
		_time = MilliSecs()
		initData()
	EndMethod
	
	Method initData()
		_keycode = GetChar()
		_mousebuttons = _d1*%1 + _d2*%10 + _d3*%100
		_mousex = _locx
		_mousey = _locy
		_mousez = _locz
	EndMethod
	
	'---------
	'accessors
	'---------
	
	Rem
	bbdoc: Returns the events type
	EndRem
	Method getType:Int()
		Return _type
	EndMethod
	
	Rem
	bbdoc: Set the events type
	EndRem
	Method setType( type_:Int )
		_type = type_
	EndMethod
	
	Method getTimestamp:Int()
		Return _time
	EndMethod
	
	Rem
	bbdoc: Returns the events keycode data
	EndRem
	Method getKeycode:Int()
		Return _keycode
	EndMethod
	
	Rem
	bbdoc: Returns the events mouse button data
	EndRem
	Method getMouseButtons:Byte()
		Return _mousebuttons
	EndMethod
	
	Rem
	bbdoc: Returns the Mouse position X
	EndRem
	Method getPositionX:Float()
		Return _mousex
	EndMethod
	
	Rem
	bbdoc: Returns the Mouse position Y
	EndRem
	Method getPositionY:Float()
		Return _mousey
	EndMethod
	
	Rem
	bbdoc: Returns the Mouse position Z
	EndRem
	Method getPositionZ:Float()
		Return _mousez
	EndMethod
	
	'-----------------
	'global accsessors
	'-----------------
	
	Rem
	bbdoc: Returns the mouse position x
	EndRem
	Function getMouseX:Int()
		Return _locx
	EndFunction
	
	Rem
	bbdoc: Returns the mouse position y
	EndRem
	Function getMouseY:Int()
		Return _locy
	EndFunction
	
	Rem
	bbdoc: Returns mouse position (StdPoint)
	EndRem
	Function getMousePosition:StdPoint()
		Return StdPoint.Create(_locx, _locy)
	EndFunction
	
	Rem
	bbdoc: Returns the mouse position delta x
	EndRem
	Function getMouseDeltaX:Int()
		Return _dlocx
	EndFunction
	
	Rem
	bbdoc: Returns the mouse position delta y
	EndRem
	Function getMouseDeltaY:Int()
		Return _dlocy
	EndFunction
	
	Rem
	bbdoc: Returns the mouse position delta z (wheel)
	EndRem
	Function getMouseDeltaWheel:Int()
		Return _dlocz
	EndFunction
	
	Rem
	bbdoc: Returns mose down left
	EndRem
	Function getMouseDownLeft:Int()
		Return _d1
	EndFunction
	
	Rem
	bbdoc: Returns mose down right
	EndRem
	Function getMouseDownRight:Int()
		Return _d2
	EndFunction
	
	Rem
	bbdoc: Returns mose down middle
	EndRem
	Function getMouseDownMiddle:Int()
		Return _d3
	EndFunction
	
	'------
	'update
	'------
	
	Function update()
		_locx = MouseX()
		_locy = MouseY()
		_locz = MouseZ()
		_dlocx = MouseXSpeed()
		_dlocy = MouseYSpeed()
		_dlocz = MouseZSpeed()
		_d1 = MouseDown(1)
		_d2 = MouseDown(2)
		_d3 = MouseDown(3)
	EndFunction	

EndType


Rem
bbdoc:
EndRem
Type UIEventManager

	Global _instance:UIEventManager
	
	Field _hadLeft:Int, _hadRight:Int, _hadKey:Int
	
	'---------
	'singleton
	'---------
	
	Function getInstance:UIEventManager()
		Return UIEventManager._instance
	EndFunction
	
	'------
	'create
	'------
	
	Function Create:UIEventManager()
		Local ret_:UIEventManager = New UIEventManager
			ret_.init()
		UIEventManager._instance = ret_
	EndFunction
	
	Method init()
	
	EndMethod
	
	'------
	'update
	'------
	
	Method update()
	
		If UIEvent.getMouseDeltaX() Or UIEvent.getMouseDeltaY()
			UIView.sendNewEvent(UIEvent.Create(UIEventMouseMoved))
		ElseIf UIEvent.getMouseDownLeft() And Not _hadLeft
			UIView.sendNewEvent(UIEvent.Create(UIEventLeftMouseDown))
			_hadLeft = True
		ElseIf UIEvent.getMouseDownRight() And Not _hadRight
			UIView.sendNewEvent(UIEvent.Create(UIEventRightMouseDown))
			_hadRight = True
		EndIf
		If Not UIEvent.getMouseDownLeft() And _hadLeft
			UIView.sendNewEvent(UIEvent.Create(UIEventLeftMouseUp))
			_hadLeft = False
		ElseIf Not UIEvent.getMouseDownRight() And _hadRight
			UIView.sendNewEvent(UIEvent.Create(UIEventRightMouseUp))
			_hadRight = False
		EndIf
		If UIEvent.getMouseDeltaWheel()
			UIView.sendNewEvent(UIEvent.Create(UIEventMouseWheel))
		EndIf	
		
		Rem
		For Local key_:Int = 37 To 40
			If KeyDown(key_) And _hadKey = 0
				Local event_:UIEvent = UIEvent.Create(UIEventKeyDown)
				event_._keycode = key_
				UIView.sendNewEvent(event_)
				_hadKey = key_
				
				Exit
			ElseIf Not KeyDown(key_)
				If _hadKey = key_
					Local event_:UIEvent = UIEvent.Create(UIEventKeyUp)
					event_._keycode = key_
					UIView.sendNewEvent(event_)
					
					_hadKey = 0
				EndIf
			EndIf
		Next
		EndRem
		
		Local char_:Int = GetChar()
		If char_
			Local event_:UIEvent = UIEvent.Create(UIEventCharacter)
			event_._keycode = char_
			UIView.sendNewEvent(event_)
		EndIf
		
	EndMethod

EndType