
Rem
bbdoc: 
EndRem
Type UIStyle Extends StdObject Abstract
	
	Global _current:UIStyle
	
	Field fontColor:VColor
	Field alternateFontColor:VColor
	
	'------------
	'Get instance
	'------------
	
	Function getCurrentStyle:UIStyle()
		Return UIStyle._current
	EndFunction
	
	Function setCurrentStyle( style_:UIStyle )
		UIStyle._current = style_
	EndFunction
	
	'------
	'create
	'------

	'----------------------------
	'abstract view render methods
	'----------------------------
	
	'windows
	Method renderWindow( window_:UIWindow ) Abstract
	
	'buttons
	Method renderButton( button_:UIButton ) Abstract
	
	'sliders
	Method renderSlider( slider_:UISlider ) Abstract
	
	'labels
	Method renderLabel( label_:UILabel ) Abstract
	
	'menu
	'Method renderMenu( menu_:UIMenu ) Abstract
	'Method renderMenuItem( item_:UIMenuItem ) Abstract

EndType

Rem
bbdoc: UI's default Style
EndRem
Type UIDefaultStyle Extends UIStyle

	Field backgroundColor:VColor
	Field alternateBackgroundColor:VColor
	Field borderColor:VColor
	Field alternateBorderColor:VColor
	
	Field gradient:VLinearGradient
	Field alternateGradient:VLinearGradient

	'------
	'create
	'------
	
	Function Create:UIDefaultStyle()
		Local ret_:UIDefaultStyle = New UIDefaultStyle
			ret_.init()
		Return ret_
	EndFunction
	
	Method init()
		backgroundColor 			= VColor.Create(.95, .95, .95, 1.0)
		alternateBackgroundColor 	= VColor.Create(0.8, 0.8, 0.8, 1.0)
		borderColor				= VColor.Create(0.5, 0.5, 0.5, 1.0)
		alternateBorderColor		= VColor.Create(115/255.0, 163/255.0, 241/255.0, 1.0)
		fontColor					= VColor.black().copy()
		alternateFontColor			= VColor.Create(0.3, 0.3, 0.3, 1.0)
		
		gradient					= VLinearGradient.Create()
								gradient.addColorStop(0.0 , 0.8, 0.8, 0.8, 1.0)
								gradient.addColorStop(1.0 , 0.6, 0.6, 0.6, 1.0)
								gradient.setOrientation(YES)
		alternateGradient			= VLinearGradient.Create()
								alternateGradient.addColorStop(0.0 , 0.7, 0.7, 0.7, 1.0)
								alternateGradient.addColorStop(1.0 , 0.5, 0.5, 0.5, 1.0)
								alternateGradient.setOrientation(YES)
	EndMethod
	
	'-------------------
	'view render methods
	'-------------------
	
	Method renderLabel( label_:UILabel )
		label_._text_color.set()
		Local x:Float
		Local y:Float
		
		Select(label_.getTextAlign())
			Case UIAlignLeft
				x = 0.0
			Case UIAlignRight
				x = label_._abs_frame.size.width-label_.getContentWidth()
			Case UIAlignCenter
				x = label_._abs_frame.size.width/2.0-label_.getContentWidth()/2.0
		EndSelect
		Select(label_.getTextVerticalAlign())
			Case UIAlignTop
				y = 0.0
			Case UIAlignBottom
				y = label_._abs_frame.size.height-label_.getContentHeight()
			Case UIAlignCenter
				y = label_._abs_frame.size.height/2.0-label_.getContentHeight()/2.0
		EndSelect
		
		DrawText(label_._text, x, y)
	EndMethod
	
	Method renderWindow( window_:UIWindow )
		gradient.drawWithStdSize(window_._titlebar_frame.size)
			
		alternateFontColor.set()
		DrawText(window_._title, window_._titlebar_frame.size.width/2.0-TextWidth(window_._title)/2.0, 4.0)
		
		backgroundColor.set()
		C2DDrawRect(0.0, window_._titlebar_frame.size.height, window_._abs_frame.size.width, window_._abs_frame.size.height - window_._titlebar_frame.size.height, C2D_AREA_FILL)
	
		borderColor.set()
		C2DDrawRect(0.0, 0.0, window_._titlebar_frame.size.width, window_._titlebar_frame.size.height, C2D_AREA_STROKE)
		C2DDrawRect(0.0, 0.0, window_._abs_frame.size.width, window_._abs_frame.size.height, C2D_AREA_STROKE)
		If window_.getresizeable() C2DDrawRect(window_._resize_frame.origin.x-window_._abs_frame.origin.x, window_._resize_frame.origin.y-window_._abs_frame.origin.y, window_._resize_frame.size.width, window_._resize_frame.size.height, C2D_AREA_STROKE)
	EndMethod
	
	Method renderButton( button_:UIButton )	
		If button_._hover Or button_._state
			If button_._hover = 2 Or button_._state
				alternateGradient.drawWithStdSize(button_._abs_frame.size)
			Else
				gradient.drawWithStdSize(button_._abs_frame.size)
			EndIf
			alternateBorderColor.set()
		Else
			gradient.drawWithStdSize(button_._abs_frame.size)
			borderColor.set()
		EndIf
		
		C2DDrawRect(0.0, 0.0, button_._abs_frame.size.width, button_._abs_frame.size.height, C2D_AREA_STROKE)
	EndMethod
	
	Method renderSlider( slider_:UISlider )
		alternateBackgroundColor.set()
		C2DDrawRect(0.0, 0.0, slider_._abs_frame.size.width, slider_._abs_frame.size.height, C2D_AREA_FILL)
	
		borderColor.set()
		C2DDrawRect(0.0, 0.0, slider_._abs_frame.size.width, slider_._abs_frame.size.height, C2D_AREA_STROKE)
		
		Select(slider_._orientation)
			Case UIOrientationHorizontal
				If slider_._hover
					If slider_._hover = 2
						alternateGradient.drawInRect(slider_._knob_pos-slider_._knob_size/2.0, 1.0, slider_._knob_size, slider_._abs_frame.size.height-2.0)
					Else
						gradient.drawInRect(slider_._knob_pos-slider_._knob_size/2.0, 1.0, slider_._knob_size, slider_._abs_frame.size.height-2.0)
					EndIf 
					alternateBorderColor.set()
				Else
					gradient.drawInRect(slider_._knob_pos-slider_._knob_size/2.0, 1.0, slider_._knob_size, slider_._abs_frame.size.height-2.0)
					borderColor.set()
				EndIf
				C2DDrawRect(slider_._knob_pos-slider_._knob_size/2.0, 0.0, slider_._knob_size, slider_._abs_frame.size.height, C2D_AREA_STROKE)
			Case UIOrientationVertical				
				If slider_._hover
					If slider_._hover = 2
						alternateGradient.drawInRect(1.0, slider_._knob_pos-slider_._knob_size/2.0, slider_._abs_frame.size.width-2.0, slider_._knob_size)
					Else
						gradient.drawInRect(1.0, slider_._knob_pos-slider_._knob_size/2.0, slider_._abs_frame.size.width-2.0, slider_._knob_size)
					EndIf 
					alternateBorderColor.set()
				Else
					gradient.drawInRect(1.0, slider_._knob_pos-slider_._knob_size/2.0, slider_._abs_frame.size.width-2.0, slider_._knob_size)
					borderColor.set()
				EndIf
				C2DDrawRect(0.0, slider_._knob_pos-slider_._knob_size/2.0, slider_._abs_frame.size.width, slider_._knob_size, C2D_AREA_STROKE)
		EndSelect
	EndMethod
	
	'Method renderMenu( menu_:UIMenu )
	'	borderColor.set()
	'	C2DDrawRect(0.0, 0.0, menu_._abs_frame.size.width, menu_._abs_frame.size.height, C2D_AREA_STROKE)
	'EndMethod
	
	'Method renderMenuItem( item_:UIMenuItem )
	'	If item_._hover 
	'		alternateBackgroundColor.set()
	'	Else
	'		backgroundColor.set()
	'	EndIf
	'	C2DDrawRect(0.0, 0.0, item_._abs_frame.size.width, item_._abs_frame.size.height, C2D_AREA_FILL)
	'	
	'	alternateFontColor.set()
	'	DrawText(item_._title, item_._abs_frame.size.width/2.0-TextWidth(item_._title)/2.0, item_._abs_frame.size.height/2.0-TextHeight(item_._title)/2.0)
	'EndMethod

EndType