

Rem
bbdoc: UIView
about: View Superclass
End Rem
Type UIView Extends UIResponder

	Global _focusView:UIView	'The currently focused view
	Global _eventView:UIView

	'---------------
	'view dimensions
	'---------------

	Field _frame:StdRect			'The size and position of the view in client coordinates
	Field _org_frame:StdRect		'The Original frame of the view. Only needed for relative autosizing
	Field _abs_frame:StdRect		'The size and position of the view in screen coordinates
	Field _visible_frame:StdRect	'The size ans position of the view if cliping is True
	Field _bounds:StdRect			'The views internal coordiante system

	'-----
	'align
	'-----

	Field _align:Int[4]		'The align of the view: left, right, top, bottom
	Field _autosize:Int[2]		'The autosize of the view: x, y
	Field _aw:Float, _ah:Float	'The distances from parents border

	'----------
	'misc flags
	'----------

	Field _modal:Int			'Tells, whether the view is modal. note: modal wont be updated normaly 
	Field _focus:Int			'Tells, whether the view has the focus
	Field _enabled:Int			'Tells, whether the view is currently enabled (receives events)
	Field _hidden:Int			'Tells, whether the view is currently visible
	Field _clip:Int 			'Tells, whether client views should be clipped on the own frame

	'-----------
	'mouse flags
	'-----------
	
	Field _hadmouse:Int		'Tells, whether the mouse collide with the view
	Field _haddown:Int			'Tells, whether the mouse clicked the view

	'--------------
	'dragging flags
	'--------------

	Field _dragmode:Int		'The dragmode (draging, resizing, ..) of the view

	'------------------------------
	'visual appearance related data
	'------------------------------

	Field _layer:VLayer		'The Views visual layer
	Field _abs_rotation:Float
	Field _style:UIStyle		'The views style
	
	'---------
	'animation
	'---------
	
	Field _animation_queue:TList

	'--------------
	'view hierarchy
	'--------------

	Field _superview:UIView	'The parent view
	Field _subviews:TList		'The views children views
	Field _link:TLink			'The link to this view in the parents children list	

	'--------------
	'public methods
	'--------------

	Rem
	bbdoc: Get the focused view
	EndRem
	Function focusView:UIView()
		Return UIView._focusView
	EndFunction

	Rem
	bbdoc: Set the focused view
	EndRem
	Function setFocusView( view_:UIView )
		UIView._focusView = view_
	EndFunction
	
	'-----
	'event
	'-----
	
	Rem
	bbdoc: Sends a event that will be dispatched and send to the right view
	EndRem
	Function sendNewEvent( event:UIEvent )
		Local view:UIView = UIView._eventView
		
		If event.getType() = UIEventMouseMoved
			If event.getMouseButtons() & UILeftMouseButton
				event.setType(UIEventLeftMouseDragged)
			ElseIf event.getMouseButtons() & UIRightMouseButton
				event.setType(UIEventRightMouseDragged)
			Else
				Local over:UIView = UIGetNullView().hitTest(StdPoint.Create(UIEvent.getMousex(), UIEvent.getMousey()))
				If view And view <> over And view._hadmouse
					event.setType(UIEventMouseLeft)
					view._hadmouse = 0
				Else
					view = over
					If view._hadmouse = 0
						event.setType(UIEventMouseEntered)
						view._hadmouse = 1
					EndIf
				EndIf
			EndIf
		ElseIf event.getType() = UIEventKeyDown Or event.getType() = UIEventKeyUp Or event.getType() = UIEventCharacter
			If UIResponder.firstResponder() UIResponder.firstResponder().dispatchEvent(event)
			view = Null
		EndIf
		
		If event.getType() = UIEventLeftMouseUp Or event.getType() = UIEventLeftMouseDown Or event.getType() = UIEventRightMouseUp Or event.getType() = UIEventRightMouseDown
			Local over:UIView = UIGetNullView().hitTest(StdPoint.Create(UIEvent.getMousex(), UIEvent.getMousey()))
			If view And view <> over And view._hadmouse
				event.setType(UIEventMouseLeft)
				view._hadmouse = 0
			Else
				view = over
				If view._hadmouse = 0
					event.setType(UIEventMouseEntered)
					view._hadmouse = 1
				EndIf
			EndIf
		EndIf
		
		If view	
			view.dispatchEvent(event)
			UIView._eventView = view
		EndIf
	EndFunction

	'------
	'create
	'------

	Rem
	bbdoc: 
	EndRem
	Function Create:UIView( frame_:StdRect )
		Local ret_:UIView = New UIView
		ret_.initWithFrame(frame_)
		Return ret_
	End Function
		
	Rem
	bbdoc: Initializes the view with a specific frame
	EndRem
	Method initWithFrame( frame_:StdRect )
		If Not frame_ Throw "Error initializing '" +typeName()+ "' with 'NULL' frame"
		
		setLayer(VLayer.Create())
		setSubviews(CreateList())
		setEnabled( True )
		setClip( True )
		setAlign(1, 0, 1, 0, 0, 0)
		setFrame( frame_ )
		setAnimationQueue(CreateList())
	EndMethod
	
	Rem
	bbdoc: Initializes the view with data
	EndRem
	Method initWithData()
		Throw "Error initializing '" +typeName()+ "' with data"
	EndMethod
	
	'---------
	'set & get
	'---------
	
	Rem
	bbdoc: Set the views frame And coordinates, relative To the superview
	about: (Copy) Calls ON_MOVE and ON_RESIZE also sets redraw to REDRAW_NEEDED
	EndRem
	Method setFrame( frame_:StdRect )
		_frame = frame_.copy()
		_org_frame = frame_.copy()
		_abs_frame = frame_.copy()
		_visible_frame = frame_.copy()
		If Not _bounds _bounds = StdRect.Create(0.0, 0.0, 0.0, 0.0)
		_bounds.size = frame_.size.copy()
		_realign()
	EndMethod
	
	Rem
	bbdoc: Returns the relative frame of the view
	EndRem
	Method getFrame:StdRect()
		Return _frame
	EndMethod
	
	Rem
	bbdoc: Returns the absolute frame of the view
	EndRem
	Method getAbsoluteFrame:StdRect()
		Return _abs_frame
	EndMethod
	
	Rem
	bbdoc: Returns the absolute rotation of the view
	about Note that the collision only uses the views own rotation, not the parent rotation 
	EndRem
	Method getAbsoluteRotation:Float()
		Return _abs_rotation
	EndMethod
	
	Rem
	bbdoc: Returns the clipping frame of the view
	about: If the view is not clipped, the absolute frame will be returned
	EndRem
	Method getVisibleFrame:StdRect()
		If getClip() = YES
			Return _visible_frame
		Else
			Return _abs_frame
		EndIf
	EndMethod
	
	Rem
	bbdoc: Set the childs offset (#StdRect)
	about: (Copy)
	EndRem
	Method setBounds( bounds_:StdRect )
		_bounds = bounds_.copy()
	EndMethod
	
	Rem
	bbdoc: Returns the views bounds (child, offset, coord scaling)
	EndRem
	Method getBounds:StdRect()
		Return _bounds
	EndMethod
	
	Rem
	bbdoc: Set the views focus To YES And releases the currently focused views focus
	about: Calls ON_FOCUS 
	EndRem
	Method setFocus()
		If Not _focus
			If UIView.focusView() Then UIView.focusView()._releaseFocus()
			UIView.setFocusView( Self )
			_focus = True
		End If
	EndMethod
	
	Rem
	bbdoc: Returns If the view is focused
	EndRem
	Method getFocus:Int()
		Return _focus
	EndMethod
	
	Rem
	bbdoc: Set If the view should be enabled
	about: Disabled views will not recieve any events
	EndRem
	Method setEnabled( enabled_:Int )
		_enabled = enabled_
	EndMethod
	
	Rem
	bbdoc: Returns If the view is enabled
	EndRem
	Method getEnabled:Int()
		Return _enabled
	EndMethod
	
	Rem
	bbdoc: Toggle If the view should be drawn Or Not
	about: Hidden views recieve events like shown views.<br>Calls ON_HIDE/ON_SHOW
	EndRem
	Method setHidden( hidden_:Int )
		_hidden = hidden_
	EndMethod
	
	Rem
	bbdoc: Returns the views hidden state
	EndRem
	Method getHidden:Int()
		Return _hidden
	EndMethod
	
	Rem
	bbdoc: Set the views parentview
	about: (Private) This method should not be called
	EndRem
	Method _setSuperview( parent_:UIView, realign_:Byte=YES )
		_superview = parent_
		setNextResponder(parent_)
		If realign_ _realign()
	EndMethod
	
	Rem
	bbdoc: Returns the views parentview
	EndRem
	Method getSuperview:UIView()
		Return _superview
	EndMethod
	
	Rem
	bbdoc: Returns the views child views
	about: The childview list is Z-Order sorted
	EndRem
	Method setSubviews( views_:TList )
		_subviews = views_
	EndMethod
	
	Rem
	bbdoc: Returns the views child views
	about: The childview list is Z-Order sorted
	EndRem
	Method getSubviews:TList()
		Return _subviews
	EndMethod
	
	Rem
	bbdoc: Set If the view should clip Or Not
	about: Clipped views will be drawn into their FBO
	EndRem
	Method setClip( clip_:Int )
		_clip = clip_
	EndMethod

	Rem
	bbdoc: Return If the view clips
	EndRem
	Method getClip:Int()
		Return _clip
	EndMethod
	
	Rem
	bbdoc: Set the gadgets alignment And autosizing propertys
	EndRem
	Method setAlign( left_:Byte=1, right_:Byte=0, top_:Byte=1, bottom_:Byte=0, autox_:Byte=0, autoy_:Byte=0 )
		_align[0] = left_
		_align[1] = right_
		_align[2] = top_
		_align[3] = bottom_
		_autosize[0] = autox_
		_autosize[1] = autoy_
	EndMethod
	
	Rem
	bbdoc: Returns all alignment And autosizing properties (by reference)
	EndRem
	Method getAlign( left_:Int Var, right_:Int Var, top_:Int Var, bottom_:Int Var, autox_:Int Var, autoy_:Int Var  )
		left_ = _align[0]
		right_ = _align[1]
		top_ = _align[2]
		bottom_ = _align[3]
		autox_ = _autosize[0]
		autoy_ = _autosize[1]
	EndMethod
	
	'------
	'visual
	'------
	
	Rem
	bbdoc: Returns the views layer instance
	EndRem
	Method getLayer:VLayer()
		Return _layer
	EndMethod

	Rem
	bbdoc: Set the views layer instance
	EndRem
	Method setLayer( layer_:VLayer )
		_layer = layer_
	EndMethod
	
	Rem
	bbdoc: Set the views rotation
	about: If the angle is <> 0 then clipping will eb disabled
	EndRem
	Method SetRotation( r_:Float=0.0 )
		If r_ <> 0.0 setClip(NO)
		_layer._rotation = r_ Mod 360
	EndMethod
	
	Rem
	bbdoc: Get the views rotation
	EndRem
	Method GetRotation:Float()
		Return _layer._rotation
	EndMethod
	
	Rem
	bbdoc: Set the animation queue
	EndRem
	Method setAnimationQueue( list_:TList )
		_animation_queue = list_
	EndMethod
	
	Rem
	bbdoc: Get the animation queue
	EndRem
	Method getAnimationQueue:TList()
		Return _animation_queue
	EndMethod
	
	'------------
	'misc get/set
	'------------
	
	Rem
	bbdoc: Retuns If the view had the mouse, last update
	EndRem
	Method _getHadMouse:Int()
		Return _hadmouse
	EndMethod
	
	Rem
	bbdoc: Retuns If the view had a mouse down, last update
	EndRem
	Method _getHadDown:Int()
		Return _haddown
	EndMethod
	
	'---------
	'animation
	'---------
	
	'Method commitAnimation( anim_:StdAnimation )
	'	anim_.setDelegate(Self)
	'	_animation_queue.addLast(anim_)
	'	If _animation_queue.count() = 1
	'		anim_.play()
	'	EndIf
	'EndMethod
	
	'Method animationForKey:StdAnimation( key_:String )
	'	Select(key_)
	'		Case "wobble"
	'			Local anim:StdFloatStepAnimation = StdFloatStepAnimation.Create()
	'				anim.setValues([-1.0, 1.0, -0.8, 0.8, -0.5, 0.5, -0.25, 0.25, -0.1, 0.1, 0.0, 0.0])
	'				anim.setFloat(Varptr _layer._rotation)
	'				anim.setFrequency(60.0)
	'				anim.setLength(0.8)
	'			Return anim
	'		Default
	'			Print "Warning: Unknown animation key '"+key_+"'"
	'			Return StdFloatAnimation.Create()
	'	EndSelect
	'EndMethod
	
	'Method animationDidStop( animation_:StdAnimation )
	'	If animation_ animation_.remove()
	'	_animation_queue.removefirst()
	'	If _animation_queue.first() StdAnimation(_animation_queue.first()).play()
	'EndMethod
	
	'-----------
	'coordinates
	'-----------
	
	Method translatePoint:StdPoint( point_:StdPoint )
		Return StdPoint.Create(point_.x-_abs_frame.origin.x, point_.y-_abs_frame.origin.y)
	EndMethod
	
	'-------------------
	'add/remove children
	'-------------------
	
	Rem
	bbdoc: Add a child view To the view
	about: It is not allowed to add the view itselfs
	EndRem
	Method addSubview( view_:UIView, realign_:Byte=True )
		VERIFY view_<>Self, "Attempt to add Self or 'NULL' as view child!"
		view_._setSuperview( Self, realign_ )
		getSubviews().addLast( view_ )
		view_._setLink getSubviews().LastLink()
	EndMethod
	
	Rem
	bbdoc: Remove a child view To the view
	EndRem
	Method removeSubview( view_:UIView )
		VERIFY view_<>Self, "Attempt to remove Self from view child!"
		view_._setSuperview( Null )
		view_._getLink().remove()
		view_._setLink Null
	EndMethod
	
	'-------
	'z order
	'-------
	
	Rem
	bbdoc: Order the view To the front
	EndRem
	Method orderFront()
		If getSuperview() And _link
			_link.remove()
			_superview._subviews.addLast(Self)
			Self._setSuperView(getSuperview(), No)
			Self._setLink(getSuperview().getSubviews().LastLink())
			getSuperview().orderFront()
		EndIf
	EndMethod
	
	Rem
	bbdoc: Order the view front And set the focus
	EndRem
	Method orderFrontAndFocus()
		orderFront()
		setFocus()
		becomeFirstResponder()
	EndMethod
	
	'--------
	'hit test
	'--------
	
	Rem
	bbdoc: Returns the topmost view on a specific position
	EndRem
	Method hitTest:UIView( location_:StdPoint )
		Local ret_:UIView = Self
		
		Local childlink_:TLink = _subviews.LastLink()
		While childlink_
			Local child_:UIView = UIView(childlink_.Value())
				
				If child_ And child_.collidesWithPoint(location_)
					ret_ = child_.hitTest(location_)
					Return ret_
				EndIf
	
			childlink_ = childlink_.PrevLink()
		Wend

		Return ret_
	EndMethod
	
	Rem
	bbdoc: Returns if the views visible frame collides with the given point<br>Does not include z-order
	about: For collision ith z-order use #hitTest
	EndRem
	Method collidesWithPoint:Int( location_:StdPoint )		
		If _layer._rotation
			Return _visible_frame.collidesWithPointRotated(_layer._rotation, _layer._rothandlex, _layer._rothandley, location_)
		Else
			Return _visible_frame.collidesWithPoint(location_)
		EndIf
	EndMethod

	'------
	'update
	'------
	
	Rem
	bbdoc: Updates the Graphics/appearance of the view And its childs
	about: (Private) Its not neccesary to call this method by hand
	EndRem
	Method updateGraphics()
		'update this views absolute size and position
		updateAbsoluteFrame()
		
		'if this view is hidden, then just ommit any drawing or recursion
		If getHidden() = YES Then Return
		
		'draw self
		draw()
		
		'get the first (backmost) child
		Local childlink_:TLink = _subviews.FirstLink()
		
		'iterate through the children, from back to top
		While childlink_
			Local child_:UIView = UIView(childlink_.Value())
			
			'set viewport if child should be clipped
			If child_.getClip() UISetViewport(child_._visible_frame) Else glDisable(GL_SCISSOR_TEST)

			'update the childs graphics
			child_.updateGraphics()
			
			childlink_ = childlink_.NextLink()
		Wend
		
		'reset viewport
		glDisable(GL_SCISSOR_TEST)
		
		_layer.pop()
	EndMethod
	
	Rem
	bbdoc: Redraws the view using its #render Method
	about: (Private) This method will be called if the views redraw state >= REDRAW_NEEDED
	EndRem
	Method draw()
		If getClip() UISetViewport(_visible_frame)
	
		_layer.push()
			render()
		'_layer.pop()
	EndMethod
	
	Rem
	bbdoc: Render the views style
	about: You dont have to translate the coordinates.
	EndRem
	Method render()
		'custom drawing here...
	EndMethod
	
	Rem
	bbdoc: Updates the absolute And the clipping frame of the view
	about: (Private) This will be called on every update
	EndRem
	Method updateAbsoluteFrame()
		_abs_rotation = _layer._rotation + _superview._abs_rotation
		_abs_frame.set(_frame)
		_layer._posx = _frame.origin.x
		_layer._posy = _frame.origin.y
		updateAlignment()
		_abs_frame.origin.x = _layer._posx
		_abs_frame.origin.y = _layer._posy
		_abs_frame.origin.add( getSuperview().getAbsoluteFrame().origin )
		_abs_frame.origin.add( getSuperview()._bounds.origin )
		
		'update the layers propertys
		_layer._width  = _abs_frame.size.width
		_layer._height = _abs_frame.size.height
		_layer._posx :+ _superview._bounds.origin.x'_abs_frame.origin.x
		_layer._posy :+ _superview._bounds.origin.y'_abs_frame.origin.y
		_layer._rothandlex = _abs_frame.size.width/2.0
		_layer._rothandley = _abs_frame.size.height/2.0
		
		If getClip() Then
			_visible_frame = _abs_frame.intersectWithRect( getSuperview()._visible_frame )	
		Else
			_visible_frame = _abs_frame
		EndIf
	EndMethod
	
	Rem
	bbdoc: Updates the alignment
	about: (Private)
	EndRem
	Method updateAlignment()
		If Not _superview Then Throw "Trying to align view of type '"+typeName()+"' without superview" ; Return 'This case must not happen. !
		
		Local aw:Float = _superview._org_frame.size.width - _frame.origin.x
		Local ah:Float = _superview._org_frame.size.height - _frame.origin.y
		
		Local l:Int, r:Int, t:Int, b:Int, ax:Int, ay:Int
		getAlign(l, r, t, b, ax, ay)
		
		If l 'left
			_layer._posx = _frame.origin.x
			If ax 'autosize on
				If r 'absolute size
					_abs_frame.size.width = getSuperview()._abs_frame.size.width - ((aw-_frame.size.width) + _frame.origin.x)
				Else 'relative size
					_abs_frame.size.width = (Self._frame.size.width / _superview._frame.size.width) * _superview._abs_frame.size.width
				EndIf
			EndIf
		ElseIf r 'right
			If ax 'autosize, relative
				_abs_frame.size.width = (Self._frame.size.width / _superview._frame.size.width) * _superview._abs_frame.size.width
				_layer._posx = getSuperview()._abs_frame.size.width - aw - (_abs_frame.size.width - _frame.size.width)
			Else
				_layer._posx = getSuperview()._abs_frame.size.width - aw
			EndIf
		Else 'cetered
			If ax 'with relative autosizin
				_abs_frame.size.width = (Self._frame.size.width / _superview._frame.size.width) * _superview._abs_frame.size.width
				_layer._posx = getSuperview()._abs_frame.size.width/2.0 - (_abs_frame.size.width/2.0)
			Else 'without relative autosizin
				_layer._posx = getSuperview()._abs_frame.size.width/2.0 - (_abs_frame.size.width/2.0)
			EndIf
		EndIf
		
		If b 'bottom
			_layer._posy = getSuperview()._abs_frame.size.height - ah
			If ay 'autosizing on
				If t 'absolute
					_layer._posy = _frame.origin.y
					_abs_frame.size.height = getSuperview()._abs_frame.size.height - ((ah-_frame.size.height) + _frame.origin.y)
				Else 'relative
					_abs_frame.size.height = (Self._frame.size.height / _superview._frame.size.height) * _superview._abs_frame.size.height
					_layer._posy = getSuperview()._abs_frame.size.height - ah - (_abs_frame.size.height - _frame.size.height)
				EndIf
			EndIf
		ElseIf t 'top
			If ay 'relative
				_abs_frame.size.height = (Self._frame.size.height / _superview._frame.size.height) * _superview._abs_frame.size.height
				_layer._posy = _frame.origin.y
			Else 'no
				_layer._posy = _frame.origin.y
			EndIf
		Else 'centered
			If ay
				_abs_frame.size.height = (Self._frame.size.height / _superview._frame.size.height) * _superview._abs_frame.size.height
				_layer._posy = getSuperview()._abs_frame.size.height/2.0 - (_abs_frame.size.height/2.0)
			Else
				_layer._posy = getSuperview()._abs_frame.size.height/2.0 - (_abs_frame.size.height/2.0)
			EndIf
		EndIf
		
		Self.alignSubviews()
	EndMethod
	
	Rem
	bbdoc: Overwrite this method for custom subview alignment or resizing
	EndRem
	Method alignSubviews()
		'...
	EndMethod
	
	'-------------
	'event methods
	'-------------

	Method leftMouseDown( event_:UIEvent )
		Self.orderFront()
	EndMethod

	'---------------
	'private methods
	'---------------
	
	Method _respondToMouseEvent:Int( event_:UIEvent )
		Return yes
	EndMethod
	
	Method _respondToKeyEvent:Int( event_:UIEvent )
		Return UIResponder.firstResponder() = Self
	EndMethod
	
	'---------------
	'private get/set
	'---------------
	
	Method _setLink( link_:TLink )
		_link = link_
	End Method
	
	Method _getLink:TLink()
		Return _link
	End Method
	
	Method _releaseFocus()
		_focus = NO
	End Method
	
	Method _realign()
		If Not _superview Return
		_org_frame = _frame.copy()
		'_aw = _superview._org_frame.size.width - _frame.origin.x
		'_ah = _superview._org_frame.size.height - _frame.origin.y
	EndMethod
	
EndType