
Rem
bbdoc:
EndRem
Const UIOrientationHorizontal:Int = 0

Rem
bbdoc:
EndRem
Const UIOrientationVertical:Int = 1

Rem
bbdoc: 
EndRem
Const UITickmarksDisabled:Int = False

Rem
bbdoc: 
EndRem
Const UITickmarksEnabled:Int = True

Rem
bbdoc: UISlider
about: A UISlider view displays a value between the range of two values
EndRem
Type UISlider Extends UIControl

	Field _min:Float
	Field _max:Float
	Field _value:Float
	
	Field _knob_pos:Float
	Field _knob_size:Float
	Field _knob_frame:StdRect
	Field _drag_point:StdPoint
	
	Field _hover:Byte
	
	Field _orientation:Byte
	Field _mousewheel:Byte
	Field _tickmarks:Byte
	
	'------
	'create
	'------
	
	Function Create:UISlider( frame_:StdRect )
		Local ret_:UISlider = New UISlider
			ret_.initWithFrame(frame_)
		Return ret_
	EndFunction
	
	Method initWithFrame( frame_:StdRect )
		Super.initWithFrame(frame_)
		
		setMinValue(0.0)
		setMaxValue(100.0)
		setValue(50.0)
		setMouseWheel(NO)
		setTickmarks(UITickmarksDisabled)
		
		_knob_pos = 0.0
		_knob_size = 32.0
		_knob_frame = StdRect.Create(0.0, 0.0, 0.0, 0.0)
		_drag_point = StdPoint.Create(0.0, 0.0)
	EndMethod
	
	'----------
	'accsessors
	'----------
	
	Rem
	bbdoc: Set the sliders min value
	EndRem
	Method setMinValue( min_:Float )
		_min = min_
	EndMethod
	
	Rem
	bbdoc: Get the sliders min value
	EndRem
	Method getMinValue:Float()
		Return _min
	EndMethod
	
	Rem
	bbdoc: Set the sliders max value
	EndRem
	Method setMaxValue( max_:Float )
		_max = max_
	EndMethod
	
	Rem
	bbdoc: Get the sliders max value
	EndRem
	Method getMaxValue:Float()
		Return _max
	EndMethod
	
	Rem
	bbdoc: Set the sliders value
	EndRem
	Method setValue( value_:Float )
		_value = value_
	EndMethod
	
	Rem
	bbdoc: Get the sliders Value
	EndRem
	Method getValue:Float()
		Return _value
	EndMethod
	
	Rem
	bbdoc: Set the views orientation
	about: [
	UIOrientationHorizontal
	UIOrientationVertical
	]
	EndRem
	Method setOrientation( orientation_:Byte )
		_orientation = orientation_
	EndMethod
	
	Rem
	bbdoc: Get the views orientation
	EndRem
	Method getOrientation:Byte()
		Return _orientation
	EndMethod
	
	Rem
	bbdoc: Set the knobsize of the slider
	about: Min: 8.0, Max: max(width, height) 
	EndRem
	Method setKnobSize( size_:Float )
		_knob_size = Min(Max(size_, 8.0), Max(_frame.size.width, _frame.size.height))
	EndMethod
	
	Rem
	bbdoc: Get the knobsize of the slider
	EndRem
	Method getKnobSize:Float()
		Return _knob_size
	EndMethod
	
	Rem
	bbdoc: Set if the slider can be set using the mousewheel
	EndRem
	Method setMouseWheel( bool_:Byte )
		_mousewheel = bool_
	EndMethod
	
	Rem
	bbdoc: Get if the slider can be set using the mousewheel
	EndRem
	Method getMouseWheel:Byte()
		Return _mousewheel
	EndMethod
	
	Rem
	bbdoc: Set if the slider should stop at tickmarks
	about: Every integer value between min and max is used as tickmark<br>If tickmarks are ON only tickmark values can be set using the slider
	EndRem
	Method setTickmarks( bool_:Byte )
		_tickmarks = bool_
	EndMethod
	
	Rem
	bbdoc: Get if the slider can be set using the mousewheel
	EndRem
	Method getTickmarks:Byte()
		Return _tickmarks
	EndMethod 

	'------
	'events
	'------
	
	Method leftMouseDragged( event_:UIEvent )	
		Select(_orientation)
			Case UIOrientationHorizontal
				_knob_pos = UIEvent.getMouseX() - _drag_point.x
			Case UIOrientationVertical
				_knob_pos = UIEvent.getMouseY() - _drag_point.y
		EndSelect
		
		calcValue()
		If getContinuously() callAction()
	EndMethod
	
	Method leftMouseDown( event_:UIEvent )
		updateKnobFrame()
		
		If _knob_frame.collidesWithPoint(UIEvent.getMousePosition())
			_drag_point.x = UIEvent.getMouseX()-_knob_pos
			_drag_point.y = UIEvent.getMouseY()-_knob_pos
		Else
			Select(_orientation)
				Case UIOrientationHorizontal
					_knob_pos = UIEvent.getMouseX() - _abs_frame.origin.x
				Case UIOrientationVertical
					_knob_pos = UIEvent.getMouseY() - _abs_frame.origin.y
			EndSelect
			
			_drag_point.x = UIEvent.getMouseX()-_knob_pos
			_drag_point.y = UIEvent.getMouseY()-_knob_pos
		EndIf
		
		_hover = 2
			
		calcValue()
		If getContinuously() callAction()
	EndMethod
	
	Method leftMouseUp( event_:UIEvent )
		If _hover = 2 _hover = 1
		
		calcValue()
		callAction()
	EndMethod
	
	Method mouseMoved( event_:UIEvent )
		updateKnobFrame()
		If _knob_frame.collidesWithPoint(UIEvent.getMousePosition())
			If _hover = 0 _hover = 1
		Else
			_hover = 0
		EndIf
	EndMethod
	
	Method mouseLeft( event_:UIEvent )
		_hover = 0
	EndMethod
	
	Method mouseWheel( event_:UIEvent )
		If _mouseWheel
			_value :+ UIEvent.getMouseDeltaWheel()
		EndIf
	EndMethod

	'------
	'render
	'------
	
	Method render()
		UIStyle.getCurrentStyle().renderSlider(Self)
	EndMethod
	
	'-------
	'private
	'-------
	
	Method calcValue()
		Local pdelta_:Float
		Local vdelta_:Float = Max(_max, _min) - Min(_max, _min)
	
		If _knob_pos < _knob_size/2.0 _knob_pos = _knob_size/2.0
		Select(_orientation)
			Case UIOrientationHorizontal
				If _knob_pos > _abs_frame.size.width - _knob_size/2.0 _knob_pos = _abs_frame.size.width - _knob_size/2.0
				pdelta_ = (_abs_frame.size.width - _knob_size/2.0) - _knob_size/2.0
			Case UIOrientationVertical
				If _knob_pos > _abs_frame.size.height- _knob_size/2.0 _knob_pos = _abs_frame.size.height- _knob_size/2.0
				pdelta_  = (_abs_frame.size.height - _knob_size/2.0) - _knob_size/2.0
		EndSelect

		If getTickmarks()
			_value = StdRoundFloat((vdelta_/pdelta_)*(_knob_pos-_knob_size/2.0))
		Else
			_value = (vdelta_/pdelta_)*(_knob_pos-_knob_size/2.0)
		EndIf
	EndMethod
	
	Method calcPosition()
		Local pdelta_:Float
		Local vdelta_:Float = Max(_max, _min) - Min(_max, _min)

		Select(_orientation)
			Case UIOrientationHorizontal
				pdelta_ = (_abs_frame.size.width - _knob_size/2.0) - _knob_size/2.0
			Case UIOrientationVertical
				pdelta_  = (_abs_frame.size.height - _knob_size/2.0) - _knob_size/2.0
		EndSelect
		_knob_pos = _value*(pdelta_/vdelta_) + _knob_size/2.0
	EndMethod
	
	Method updateAbsoluteFrame()
		Super.updateAbsoluteFrame()
		
		calcPosition()
	EndMethod
	
	Method updateKnobFrame()
		_knob_frame.set(_abs_frame)
		Select(_orientation)
			Case UIOrientationHorizontal
				_knob_frame.origin.x :+ _knob_pos-_knob_size/2.0 
				_knob_frame.size.width = _knob_size
			Case UIOrientationVertical
				_knob_frame.origin.y :+ _knob_pos-_knob_size/2.0
				_knob_frame.size.height = _knob_size
		EndSelect
	EndMethod
EndType

Rem
bbdoc: UICircularSlider
about: A UICircularSlider view displays a value between the range of two values
EndRem
Type UICircularSlider Extends UISlider

	'... not finished yet ...

EndType