
Private
Const DRAG_MOVE	:Byte = 1
Const DRAG_RESIZE	:Byte = 2
Public

Rem
bbdoc: UIWindow
about: UIWindow provides a simple container view that is draggable and resizeable
EndRem
Type UIWindow Extends UIView {delegate="windowDidClose, windowDidMove, windowDidResize, windowWillMove, windowWillResize"}

	Field _title:String
	
	Field _titlebar_frame:StdRect
	Field _resize_frame  :StdRect
	
	Field _draggable:Byte
	Field _resizeable:Byte
	Field _dragmode:Byte
	Field _drag_point:StdPoint
	
	Field _min_size:StdSize
	Field _max_size:StdSize
	
	'buffered extension
	Field _buffered:Int
	Field _framebuffer:VFBO
	Field _buffer:TImage
	Field _rebuffer:Int
	
	'------
	'create
	'------
	
	Function Create:UIWindow( frame_:StdRect )
		Local ret_:UIWindow = New UIWindow
			ret_.initWithFrame(frame_)
		Return ret_
	EndFunction
	
	Method initWithFrame( frame_:StdRect )
		Super.initWithFrame(frame_)
		
		_resize_frame   	= StdRect.Create(0.0, 0.0, 16.0, 16.0)
		_titlebar_frame 	= StdRect.Create(0.0, 0.0, 1.0, 24.0)
		_drag_point 	 	= StdPoint.Create(0.0, 0.0)
		
		setResizeable(YES)
		setDraggable(YES)
		setTitlebarHeight(20.0)
		setTitle("untitled")
		setMinSize(StdSize.Create(44.0, 22.0))
		setMaxSize(StdSize.Create(8000.0, 8000.0))
		setBuffered(NO)
	EndMethod
	
	'----------
	'accsessors
	'----------
	
	Rem
	bbdoc: Set the windows title
	EndRem
	Method setTitle( title_:String )
		_title = title_
	EndMethod
	
	Rem
	bbdoc: Returns the windows title
	EndRem
	Method getTitle:String()
		Return _title
	EndMethod
	
	Rem
	bbdoc: Set draggable
	about: Get using #getDraggable
	EndRem
	Method setDraggable( draggable_:Byte )
		_draggable = draggable_
	EndMethod
	
	Rem
	bbdoc: Returns draggable
	about: Set using #setDraggable
	EndRem
	Method getDraggable:Byte()
		Return _draggable
	EndMethod
	
	Rem
	bbdoc: Set draggable
	about: Get using #getResizeable
	EndRem
	Method setResizeable( resizeable_:Byte )
		_resizeable = resizeable_
	EndMethod
	
	Rem
	bbdoc: Returns draggable
	about: Set using #setResizeable
	EndRem
	Method getResizeable:Byte()
		Return _resizeable
	EndMethod
	
	Rem
	bbdoc: Set the titlebars height
	EndRem
	Method setTitlebarHeight( height_:Float )
		height_ = Max(height_, 12.0)
		_titlebar_frame.size.height = height_
		setBounds(StdRect.Create(0.0, height_, 0.0, 0.0))
	EndMethod
	
	Rem
	bbdoc: Returns the titlebars height
	EndRem
	Method getTitlebarHeight:Float()
		Return _titlebar_frame.size.height
	EndMethod
	
	Rem
	bbdoc: Set the windows minimal size
	EndRem
	Method setMinSize( size_:StdSize )
		_min_size = size_.copy()
	EndMethod
	
	Rem
	bbdoc: Get the windows minimal size
	EndRem
	Method getMinSize:StdSize()
		Return _min_size
	EndMethod
	
	Rem
	bbdoc: Set the windows maximum size
	EndRem
	Method setMaxSize( size_:StdSize )
		_max_size = size_.copy()
	EndMethod
	
	Rem
	bbdoc: Get the windows maximum size
	EndRem
	Method getMaxSize:StdSize()
		Return _max_size
	EndMethod

	Rem
	bbdoc: Set that the window should be buffered in background
	about: Default is NO
	EndRem
	Method setBuffered( buffered_:Int )
		_buffered = buffered_
		If _buffered
			_buffer = CreateImage(_frame.size.width, _frame.size.height)
			_framebuffer = VFBO.withTImage(_buffer)
		EndIf
	EndMethod

	Rem
	bbdoc: Returns if the window is buffered
	EndRem
	Method getBuffered:Int()
		Return _buffered
	EndMethod

	'-----
	'event
	'-----
	
	Method leftMouseDragged( event_:UIEvent )
		Select(_dragmode)
			Case DRAG_MOVE
				If getDraggable() = NO Return
				_frame.origin.x = UIEvent.getMouseX() - _drag_point.x
				_frame.origin.y = UIEvent.getMouseY() - _drag_point.y
			Case DRAG_RESIZE
				If getResizeable() = NO Return
				_frame.size.width  = UIEvent.getMouseX() - _drag_point.x
				_frame.size.height = UIEvent.getMouseY() - _drag_point.y
				checkSize()
		EndSelect
	EndMethod
	
	Method leftMouseDown( event_:UIEvent )
		orderFront()
		
		If _resize_frame.collidesWithPoint(UIEvent.getMousePosition())
			_dragmode = DRAG_RESIZE
			_drag_point.x = UIEvent.getMouseX() - _abs_frame.size.width
			_drag_point.y = UIEvent.getMouseY() - _abs_frame.size.height
			
			callDelegate("windowWillMove")
		ElseIf _titlebar_frame.collidesWithPoint(UIEvent.getMousePosition())
			_dragmode = DRAG_MOVE
			_drag_point.x = UIEvent.getMouseX() - _abs_frame.origin.x
			_drag_point.y = UIEvent.getMouseY() - _abs_frame.origin.y
			
			callDelegate("windowWillResize")
		Else
			_dragmode = 0
		EndIf
	EndMethod
	
	Method leftMouseUp( event_:UIEvent )
		Select(_dragmode)
			Case DRAG_MOVE
				callDelegate("windowDidMove")
			Case DRAG_RESIZE
				callDelegate("windowDidResize")
		EndSelect
		_dragmode = 0
	EndMethod
	
	'------
	'render
	'------
	
	Method render()
		If _rebuffer _framebuffer.bind() 
		UIStyle.getCurrentStyle().renderWindow(Self)
		If _rebuffer 
			_framebuffer.unbind()
			_rebuffer = NO
		EndIf
	EndMethod
	
	'-------
	'private
	'-------
	
	Method updateAbsoluteFrame()
		Super.updateAbsoluteFrame()
		
		updateTitlebarFrame()
		updateResizeFrame()
	EndMethod
	
	Method updateTitlebarFrame()
		_titlebar_frame.origin.set(_abs_frame.origin)
		_titlebar_frame.size.width = _abs_frame.size.width	
	EndMethod
	
	Method updateResizeFrame()
		_resize_frame.origin.x = _abs_frame.maxx()-_resize_frame.size.width
		_resize_frame.origin.y = _abs_frame.maxy()-_resize_frame.size.height
	EndMethod
	
	Method updateAlignment()
	EndMethod
	
	Method checkSize()
		If _frame.size.width  < _min_size.width _frame.size.width  = _min_size.width
		If _frame.size.height < _min_size.height _frame.size.height = _min_size.height
		If _frame.size.width  > _max_size.width _frame.size.width  = _max_size.width
		If _frame.size.height > _max_size.height _frame.size.height = _max_size.height
	EndMethod
	
	Method getIsFront:Int()
		Return UI.getInstance().getNullView().getSubviews().lastLink() = Self._link
	EndMethod

EndType
